/*

Svar på delfråga (a):


*/

#include "wrap/thread.h"
#include "wrap/synch.h"
#include "wrap/atomics.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>

// Representerar en produkt som finns i affären.
struct product {
	// Namn på produkten. Om namnet är NULL är platsen ledig.
	const char *name;

	// Antal av denna produkt som finns i lagret.
	int in_stock;

	// Lås för denna produkten.
	struct lock stock_lock;
};

// Antal produkter i 'products' nedan.
int num_products;

// Alla produkter i affären för närvarande.
struct product *products;

// Hjälpfunktion för att hitta en produkt.
int find_product(const char *name) {
	for (int i = 0; i < num_products; i++) {
		if (!products[i].name)
			continue;

		// Jämför strängarna. Om de är lika, returnera 'i'.
		// Se 'man strcmp' för detaljer.
		if (strcmp(products[i].name, name) == 0)
			return i;
	}

	return -1;
}

// En kund vill köpa två olika produkter, 'buy1' och
// 'buy2'. Enligt din affärsidé får du bara sälja om du har båda
// produkterna i lager.
bool buy(const char *buy1, const char *buy2) {
	int id1 = find_product(buy1);
	int id2 = find_product(buy2);

	if (id1 < 0 || id2 < 0 || id1 == id2)
		return false;

	lock_acquire(&products[id1].stock_lock);
	lock_acquire(&products[id2].stock_lock);

	bool ok = products[id1].in_stock > 0
		&& products[id2].in_stock > 0;

	if (ok) {
		products[id1].in_stock--;
		products[id2].in_stock--;
	}

	lock_release(&products[id1].stock_lock);
	lock_release(&products[id2].stock_lock);

	return ok;
}


/**
 * Huvudprogram. Endast för att visa hur vi kan köra
 * programmet. Vid rättning kommer eventuella modifikationer
 * härifrån och nedåt att ignoreras.
 */

void setup_products(void) {
	num_products = 3;
	products = malloc(sizeof(struct product) * num_products);
	products[0].name = "Kaffe";
	products[0].in_stock = 2;
	products[1].name = "Te";
	products[1].in_stock = 2;
	products[2].name = "Bulle";
	products[2].in_stock = 3;

	for (int i = 0; i < num_products; i++)
		lock_init(&products[i].stock_lock);
}

void do_buy(const char *a, const char *b) {
	const char *ok = "false";
	if (buy(a, b))
		ok = "true";
	printf("Köpte %5s och %5s: %5s\n", a, b, ok);
}

int main(void) {
	setup_products();

	do_buy("Kaffe", "Bulle");
	do_buy("Te", "Bulle");
	do_buy("Kaffe", "Bulle");
	do_buy("Te", "Bulle");

	return 0;
}
